@@ -5,6 +5,8 @@ from os import environ
 
 from psutil import Process, cpu_count, cpu_times
 
+from .cgroups import get_cgroups_cpu_quota
+
 # How often to poll the CPU, compared to the moving-average of the job-start-interval
 CPU_IDLE_POLL_DIVISOR = 20
 
@@ -87,13 +89,11 @@ class WorkerList:
             return sum([w.cpu_reservation_amount for w in self.workers])
 
 
-"""Timed measurement on an incrementing counter
-
-Reads the counter, divides with time passed since last reading, and smooths the value using a moving average
-"""
-
-
 class InertialTimeDerivate:
+    """Timed measurement on an incrementing counter
+
+    Reads the counter, divides with time passed since last reading, and smooths the value using a moving average
+    """
     def __init__(self, func, initial, inertia):
         self.func = func
         self.last_value = self.func()
@@ -142,27 +142,20 @@ def wait_for_idle_cpus(
         times = process.cpu_times()
         return times.user + times.system + times.children_user + times.children_system
 
-    # CPU.limits enforced by cgroup, configured by I.E. K8S
-    # Authorative docs at https://www.kernel.org/doc/Documentation/scheduler/sched-bwc.txt
-    with open("/sys/fs/cgroup/cpu/cpu.cfs_quota_us", "rt") as quota, open(
-        "/sys/fs/cgroup/cpu/cpu.cfs_period_us", "rt"
-    ) as period:
-        quota = int(quota.read())
-        if quota > 0:
-            cpu_quota = quota / int(period.read())
-            if not max_concurrent:
-                max_concurrent = ceil(
-                    cpu_quota * DEFAULT_MAX_CONCURRENT_MULTIPLIER / required
-                )
+    cpu_quota = get_cgroups_cpu_quota()
+    if cpu_quota < inf:
+        if not max_concurrent:
+            max_concurrent = ceil(
+                cpu_quota * DEFAULT_MAX_CONCURRENT_MULTIPLIER / required
+            )
 
-            # Precalculate the quota with headroom for a new job
-            cpu_quota -= required
-        else:
-            cpu_quota = inf
-            if not max_concurrent:
-                max_concurrent = ceil(
-                    cpu_count() * DEFAULT_MAX_CONCURRENT_MULTIPLIER / required
-                )
+        # Precalculate the quota with headroom for a new job
+        cpu_quota -= required
+    else:
+        if not max_concurrent:
+            max_concurrent = ceil(
+                cpu_count() * DEFAULT_MAX_CONCURRENT_MULTIPLIER / required
+            )
 
     def cpu_is_available(idle, cpu_used):
         reserved = worker_list.reserved_cpu_amount()
