@@ -58,7 +58,7 @@ class Handler(WSGIRequestHandler):
     ):
         self.cb_request_received = cb_request_received
         self.env = None
-        self.uuid = uuid4()
+        self.request_id = None
         self.request_received_at = None
 
         with DeferrableCleanup(cb_processing_done) as c:
@@ -90,6 +90,9 @@ class Handler(WSGIRequestHandler):
 
     def parse_request(self):
         success = super().parse_request()
+        if success:
+            self.request_received_at = monotonic()
+            self.request_id = self.headers.get('X-Request-Id', None) or str(uuid4())
 
         # This socket shall no longer be fast-closed on SIGTERM
         socket = self.request
@@ -98,22 +101,21 @@ class Handler(WSGIRequestHandler):
         return success
 
     def log_message(self, format, *args):
-        LOGGER.info("[%s] " + format, self.uuid, *args)
+        LOGGER.info("[%s] " + format, self.request_id, *args)
 
     def log_request(self, code="-", size="-"):
-        self.request_received_at = monotonic()
         if isinstance(code, HTTPStatus):
             code = code.value
         LOGGER.info(
-            '[%s] "%s" %s %s', self.uuid, self.requestline, str(code), str(size)
+            '[%s] "%s" %s %s', self.request_id, self.requestline, str(code), str(size)
         )
 
     def log_error(self, format, *args):
-        LOGGER.error("[%s] " + format, self.uuid, *args)
+        LOGGER.error("[%s] " + format, self.request_id, *args)
 
     def log_elapsed(self):
         LOGGER.info(
-            "[%s] Elapsed: %s" % (self.uuid, monotonic() - self.request_received_at)
+            "[%s] Elapsed: %.3f", self.request_id, monotonic() - self.request_received_at
         )
 
 
@@ -213,13 +215,20 @@ class Server:
 
             try:
                 h = Handler(s, addr, self, cb_request_received, cb_processing_done)
-                if h.request_received_at:
-                    h.log_elapsed()
             except BaseException:
                 LOGGER.exception("In WSGI request handling")
                 sleep(1)
                 return
 
+            try:
+                s.shutdown(SHUT_RDWR)
+            except OSError:
+                # We shutdown to ensure flushing, and for courtesy, but we don't care about errors here
+                pass
+
+            if h.request_received_at:
+                h.log_elapsed()
+
     def get_app(self):
         return self.app
 
